/* Web_Demo.pde -- sample code for Webduino server library */

/*
 * To use this demo,  enter one of the following USLs into your browser.
 * Replace "host" with the IP address assigned to the Arduino.
 *
 * http://host/
 * http://host/json
 *
 * This URL brings up a display of the values READ on digital pins 0-9
 * and analog pins 0-5.  This is done with a call to defaultCmd.
 * 
 * 
 * http://host/form
 *
 * This URL also brings up a display of the values READ on digital pins 0-9
 * and analog pins 0-5.  But it's done as a form,  by the "formCmd" function,
 * and the digital pins are shown as radio buttons you can change.
 * When you click the "Submit" button,  it does a POST that sets the
 * digital pins,  re-reads them,  and re-displays the form.
 * 
 */

#include "SPI.h"
#include "Ethernet.h"
#include "WebServer.h"
#include "SD.h"

// no-cost stream operator as described at 
// http://sundial.org/arduino/?page_id=119
template<class T>
inline Print &operator <<(Print &obj, T arg)
{ 
  obj.print(arg); 
  return obj; 
}


// CHANGE THIS TO YOUR OWN UNIQUE VALUE
static uint8_t mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

// CHANGE THIS TO MATCH YOUR HOST NETWORK
static uint8_t ip[] = { 
  192, 168, 1, 178 };

#define PREFIX ""

int plant_bed_pin = 31;
int plant_bed_pin_led = 30;

int worm_bed_pin = 33;
int worm_bed_pin_led = 32;

int light_pin = 35; 
int light_pin_led = 34; 

int fan_pin = 37;
int fan_pin_led = 36;

boolean autoControl = false;



// Set up some globals for the data collection off A8
#define data_size 10
int data[data_size];
int data_index;
int max_meter_length = 20;

WebServer webserver(PREFIX, 80);

// commands are functions that get called by the webserver framework
// they can read any posted data from client, and they output to server


int post_key_value_branch(char* a_posted_key, char* a_posted_value)
{
  if ( a_posted_key == NULL) return 0;
  if ( a_posted_value == NULL ) return 0;

  if (strcmp("fan", a_posted_key) == 0)
  {
    if (strcmp("on", a_posted_value) == 0)
    {
      digitalWrite(fan_pin, HIGH);  
      digitalWrite(fan_pin_led, HIGH);  
      Serial.println(" Command --- Turning the fan ON");    
    }
    else
    {
      digitalWrite(fan_pin, LOW);    
      digitalWrite(fan_pin_led, LOW);    
      Serial.println(" Command --- Turning the fan OFF");  
    }    
  }
  else if (strcmp("light", a_posted_key) == 0)
  {
    if (strcmp("on", a_posted_value) == 0)
    {
      digitalWrite(light_pin, HIGH);  
      digitalWrite(light_pin_led, HIGH);  
      Serial.println(" Command --- Turning the light ON");    
    }
    else
    {
      digitalWrite(light_pin, LOW);   
      digitalWrite(light_pin_led, LOW);    
      Serial.println(" Command --- Turning the light OFF");  
    }    
  }  
  else if (strcmp("auto", a_posted_key) == 0)
  {
    if (strcmp("on", a_posted_value) == 0)
    {
      digitalWrite(worm_bed_pin, LOW);  
      digitalWrite(worm_bed_pin_led, LOW);  
      autoControl = true;   
    }
    else
    {
      autoControl = false; 
    }    
  }  
  else if (strcmp("worm", a_posted_key) == 0)
  {
    if (!autoControl)
    {
      if (strcmp("on", a_posted_value) == 0)
      {
        digitalWrite(worm_bed_pin, HIGH); 
        digitalWrite(worm_bed_pin_led, HIGH);  
        Serial.println(" Command --- Turning the wormbed ON");    
      }
      else
      {
        digitalWrite(worm_bed_pin, LOW); 
        digitalWrite(worm_bed_pin_led, LOW);    
        Serial.println(" Command --- Turning the wormbed OFF");  
      }    
    }
  }  
  else if (strcmp("plant", a_posted_key) == 0)
  {
    if (!autoControl)
    {
      if (strcmp("on", a_posted_value) == 0)
      {
        digitalWrite(plant_bed_pin, HIGH); 
        digitalWrite(plant_bed_pin_led, HIGH);  
        Serial.println(" Command --- Turning the plantbed ON");    
      }
      else
      {
        digitalWrite(plant_bed_pin, LOW); 
        digitalWrite(plant_bed_pin_led, LOW);    
        Serial.println(" Command --- Turning the plantbed OFF");  
      }   
    } 
  }  

  return 1; 
}



void print_meter(WebServer &server, int value)
{
  int percent = (int)(value*100 / 500);
  int meter_length = (percent * max_meter_length)/ 100;


  // server << "<pre>             [";

  for (int index = 0; index <= meter_length; index++)
  {
    server << "#";
  }


  /*
  for (int index = 0; index <= max_meter_length - meter_length; index++)
   {
   server << " ";
   }
   
   server << "]</pre>";
   */
}




void outputPins(WebServer &server, WebServer::ConnectionType type, bool addControls = false)
{
  P(htmlHead) =
    "<html>"
    "<head>"
    "<title>Arduino Web Server</title>"
    "<style type=\"text/css\">"
    "BODY { font-family: sans-serif }"
    "H1 { font-size: 14pt; text-decoration: underline }"
    "P  { font-size: 10pt; }"
    "</style>"
    "</head>"
    "<body>";

  int i;
  server.httpSuccess();
  server.printP(htmlHead);

  if (addControls)
    server << "<form action='" PREFIX "/form' method='post'>";

  // Write out a list of the data points
  server << "</p><h1>Data log</h1><p>";
  for (int index = 0; index <= data_size - 1; index++)
  {
    print_meter(server, data[index]);
    server << "                   data[" << index << "] = " << data[index] << "<br>";
  }  

  if (autoControl) server << "<br>Auto water control is ON<br>";
  else server << "<br>Auto water control is OFF<br>";

  server << "<h1>Digital Pins</h1><p>";

  for (i = 0; i <= 10; ++i)
  {
    // ignore the pins we use to talk to the Ethernet chip
    int val = digitalRead(i);
    server << "Digital " << i << ": ";
    if (addControls)
    {
      char pinName[4];
      pinName[0] = 'd';
      itoa(i, pinName + 1, 10);
      server.radioButton(pinName, "1", "On", val);
      server << " ";
      server.radioButton(pinName, "0", "Off", !val);
    }
    else
      server << (val ? "HIGH" : "LOW");

    server << "<br/>";
  }

  server << "<br/>";


  int val = digitalRead(plant_bed_pin);
  server << "Digital " << plant_bed_pin << " (plant_bed_pin): ";
  server << (val ? "HIGH<br>" : "LOW<br>");
  val = digitalRead(plant_bed_pin_led);
  server << "Digital " << plant_bed_pin_led << " (plant_bed_pin_led): ";
  server << (val ? "HIGH<br>" : "LOW<br>");

  val = digitalRead(worm_bed_pin);
  server << "Digital " << worm_bed_pin << " (worm_bed_pin): ";
  server << (val ? "HIGH<br>" : "LOW<br>");
  val = digitalRead(worm_bed_pin_led);
  server << "Digital " << worm_bed_pin_led << " (worm_bed_pin_led): ";
  server << (val ? "HIGH<br>" : "LOW<br>");

  val = digitalRead(fan_pin);
  server << "Digital " << fan_pin << " (fan_pin): ";
  server << (val ? "HIGH<br>" : "LOW<br>");
  val = digitalRead(fan_pin_led);
  server << "Digital " << fan_pin_led << " (fan_pin_led): ";
  server << (val ? "HIGH<br>" : "LOW<br>");

  val = digitalRead(light_pin);
  server << "Digital " << light_pin << " (light_pin): ";
  server << (val ? "HIGH<br>" : "LOW<br>");
  val = digitalRead(light_pin_led);
  server << "Digital " << light_pin_led << " (light_pin_led): ";
  server << (val ? "HIGH<br>" : "LOW<br>");

  //  Taken out for now
  for (i = 22; i <= 17; ++i)
  {
    // ignore the pins we use to talk to the Ethernet chip
    int val = digitalRead(i);
    server << "Digital " << i << ": ";
    if (addControls)
    {
      char pinName[4];
      pinName[0] = 'd';
      itoa(i, pinName + 1, 10);
      server.radioButton(pinName, "1", "On", val);
      server << " ";
      server.radioButton(pinName, "0", "Off", !val);
    }
    else
      server << (val ? "HIGH<br>" : "LOW<br>");

    server << "<br/>";
  }

  server << "</p><h1>Analog Pins</h1><p>";
  int plant_moisture_value = 0;
  int worm_moisture_value = 0;
  int temp_value = 0;
  for (i = 0; i <= 15; ++i)
  {
    val = analogRead(i);
    
    server << "Sensor " << i << ": " << val;

    if (i == 8)
    {
      server << "     --------- This is a temp sensor";
      temp_value += analogRead(i);
    }
    if (i == 9)
    {
      server << "     --------- This is a temp sensor";
      temp_value += analogRead(i);
    }  
    if (i == 11)
    {   
      server << "     --------- This is a plant temp sensor";
      plant_moisture_value += analogRead(i);
    }
    if (i == 12)
    { 
      server << "     --------- This is a plant temp sensor";
      plant_moisture_value += analogRead(i);
    }
    if (i == 13)
    {  
      server << "     --------- This is a plant temp sensor";
      plant_moisture_value += analogRead(i);
    }
    if (i == 14)
    {  
      server << "     --------- This is a plant temp sensor";
      plant_moisture_value += analogRead(i);
    }
    if (i == 15)
    { 
      server << "     --------- This is a worm temp sensor";
      worm_moisture_value += analogRead(i);
    }

    server << "<br/>";
  }

  temp_value = temp_value/2;
  plant_moisture_value = plant_moisture_value/4;
  worm_moisture_value = worm_moisture_value/1;

  server << "<BR>";
  server << "     Temperature value is " << temp_value << "<BR>";

  server << "     Plant soil moisture value is " << plant_moisture_value;
  if (plant_moisture_value < 25) server << "     --------- Found the plant soil to be dry." << "<BR>";
  else if (plant_moisture_value <= 200) server << "     --------- Found the plant soil to be mildly moist." << "<BR>";
  else server << "     --------- Found the plant soil to be saturated." << "<BR>";

  server << "     Worm soil moisture value is " << worm_moisture_value;
  if (worm_moisture_value < 25) server << "     --------- Found the worm soil to be dry." << "<BR>";
  else if (worm_moisture_value <= 200) server << "     --------- Found the worm soil to be mildly moist." << "<BR>";
  else server << "     --------- Found the worm soil to be saturated." << "<BR>";

  server << "<BR>";

  server << "</p>";

  if (addControls)
    server << "<input type='submit' value='Submit'/></form>";

  server << "</body></html>";
}

void formCmd(WebServer &server, WebServer::ConnectionType type, char *url_tail, bool tail_complete)
{
  if (type == WebServer::POST)
  {
    bool repeat;
    char name[16], value[16];
    do
    {
      repeat = server.readPOSTparam(name, 16, value, 16);
      if (name[0] == 'd')
      {
        int pin = strtoul(name + 1, NULL, 10);
        int val = strtoul(value, NULL, 10);
        digitalWrite(pin, val);
      }
    } 
    while (repeat);

    server.httpSeeOther(PREFIX "/form");
  }
  else
    outputPins(server, type, true);
}

void defaultCmd(WebServer &server, WebServer::ConnectionType type, char *url_tail, bool tail_complete)
{
  if (type == WebServer::POST)
  {
    bool result;
    char name[32], value[32];
    do
    {
      /* readPOSTparam returns false when there are no more parameters
       * to read from the input.  We pass in buffers for it to store
       * the name and value strings along with the length of those
       * buffers. */
      result = server.readPOSTparam(name, 32, value, 32);

      if (result) post_key_value_branch(name, value);
    } 
    while (result);

    // after procesing the POST data, tell the web browser to reload
    // the page using a GET method. 
    // server.httpSeeOther(PREFIX);
  }

  outputPins(server, type, false);  
}














void setup()
{
  // or initialize
  SD.begin(4);
  Serial.begin(9600);
  Serial.println(" Ready");

  // set up the data array and clear it out
  data_index = 0;

  for (int index = 0; index <= data_size - 1; index++)
  {
    data[index] = -1;
  }

  // set pins 0-8 for digital input
  for (int i = 0; i <= 9; ++i)
    pinMode(i, INPUT);
  pinMode(9, OUTPUT);

  Ethernet.begin(mac, ip);
  webserver.begin();

  webserver.setDefaultCommand(&defaultCmd);

  for (int index = 20; index <= 40; index++) pinMode(index, OUTPUT); 
}



void loop()
{
  // process incoming connections one at a time forever
  webserver.processConnection();  

  // Shift the data values
  for (int index = 1; index <= data_size - 1; index++)
  {
    data[index - 1] = data[index];
  }

  // Add in the new value to the end
  data[data_size- 1] = analogRead(8);


  // fix the data index if it's past the end of array
  //   data_index = data_index % data_size;

  // Sleep for a second so there is a little bit between data points
  delay(250);

  int plant_moisture_value = (analogRead(11) + analogRead(12) + analogRead(13) + analogRead(14) + analogRead(15)) / 4;
  int worm_moisture_value = analogRead(15);  
  int temp_value = (analogRead(8) + analogRead(9))/2;

  // Handle the plant watering 
  if ((autoControl == true) && (plant_moisture_value < 70))
  {
    // Dry 
    digitalWrite(plant_bed_pin, HIGH);
    digitalWrite(plant_bed_pin_led, HIGH);

    delay(500); 
  }
  else if (autoControl == true)
  {
    // Wet
    digitalWrite(plant_bed_pin, LOW);
    digitalWrite(plant_bed_pin_led, LOW);    
  }
  
  // Handle the worm watering 
  if ((autoControl == true) && (worm_moisture_value < 70))
  {
    // Dry 
    digitalWrite(worm_bed_pin, HIGH);
    digitalWrite(worm_bed_pin_led, HIGH);

    delay(500); 
  }
  else if (autoControl == true)
  {
    // Wet
    digitalWrite(worm_bed_pin, LOW);
    digitalWrite(worm_bed_pin_led, LOW);    
  }

  // if you wanted to do other work based on a connecton, it would go here
}











